@@ -1,4 +1,5 @@
 #include <algorithm>
+#include <cstddef>
 #include <fstream>
 #include <iostream>
 #include <numeric>
@@ -142,7 +143,7 @@ void show_help() {
   printf(" See https://github.com/lemire/base64data for test data.\n");
 }
 void pretty_print(size_t, size_t bytes, std::string name, event_aggregate agg) {
-  printf("%-40s : ", name.c_str());
+  printf("%-45s : ", name.c_str());
   fflush(stdout);
   double avgspeed = bytes / agg.elapsed_ns();
   double bestspeed = bytes / agg.best.elapsed_ns();
@@ -267,6 +268,7 @@ class Application {
 
   std::vector<char> buffer1;
   std::vector<char> buffer2;
+  std::vector<char> buffer3;
 
   std::vector<std::string> implementations;
 
@@ -287,6 +289,7 @@ public:
 
       buffer1.resize(simdutf::base64_length_from_binary(max_size));
       buffer2.resize(max_size);
+      buffer3.resize(simdutf::base64_length_from_binary_with_lines(max_size));
     }
   }
 
@@ -387,6 +390,7 @@ private:
       if (!e->supported_by_runtime_system()) {
         continue;
       }
+      simdutf::get_active_implementation() = e;
       summarize("simdutf::" + e->name(), [this, &e]() {
         for (const std::vector<char> &source : data) {
           size_t base64_size =
@@ -409,13 +413,19 @@ private:
     if (benchmark_mode != BenchmarkMode::list) {
       printf("# encode\n");
     }
+    volatile size_t base64_size;
 
     summarize("memcpy", [this]() {
       for (const std::vector<char> &source : data) {
         memcpy(buffer1.data(), source.data(), source.size());
       }
     });
-    volatile size_t base64_size;
+    summarize("node", [this, &base64_size]() {
+      for (const std::vector<char> &source : data) {
+        base64_size = node::base64_encode(source.data(), source.size(),
+                                          buffer1.data(), buffer1.size());
+      }
+    });
     summarize("libbase64", [this, &base64_size]() {
       for (const std::vector<char> &source : data) {
         size_t outlen;
@@ -427,12 +437,31 @@ private:
       if (!e->supported_by_runtime_system()) {
         continue;
       }
-      summarize("simdutf::" + e->name(), [this, &e, &base64_size]() {
+      simdutf::get_active_implementation() = e;
+      summarize("simdutf::" + e->name() + "_standard", [this, &e,
+                                                        &base64_size]() {
         for (const std::vector<char> &source : data) {
           base64_size =
               e->binary_to_base64(source.data(), source.size(), buffer1.data());
         }
       });
+      summarize("simdutf::" + e->name() + "_with_lines",
+                [this, &e, &base64_size]() {
+                  for (const std::vector<char> &source : data) {
+                    base64_size = e->binary_to_base64_with_lines(
+                        source.data(), source.size(), buffer3.data());
+                  }
+                });
+#if SIMDUTF_COMPILED_CXX_VERSION >= 20
+      summarize("simdutf::atomic_binary_to_base64_" +
+                    (simdutf::get_active_implementation() = e)->name(),
+                [this, &base64_size]() {
+                  for (const std::vector<char> &source : data) {
+                    base64_size = simdutf::atomic_binary_to_base64(
+                        source.data(), source.size(), buffer1.data());
+                  }
+                });
+#endif
     }
   }
 
@@ -447,7 +476,7 @@ private:
       if (!e->supported_by_runtime_system()) {
         continue;
       }
-
+      simdutf::get_active_implementation() = e;
       summarize("simdutf::" + e->name(), [this, &e]() {
         for (const std::vector<char> &source : data) {
           size_t base64_size =
@@ -532,6 +561,7 @@ private:
       if (!e->supported_by_runtime_system()) {
         continue;
       }
+      simdutf::get_active_implementation() = e;
 
       summarize("simdutf::" + e->name(), [this, &e]() {
         for (const std::vector<char> &source : data) {
@@ -569,6 +599,30 @@ private:
           }
         }
       });
+
+#if SIMDUTF_COMPILED_CXX_VERSION >= 20
+      summarize("simdutf::atomic_base64_to_binary_" +
+                    (simdutf::get_active_implementation() = e)->name(),
+                [this]() {
+                  for (const std::vector<char> &source : data) {
+                    size_t len = buffer1.size();
+                    auto err = simdutf::atomic_base64_to_binary_safe(
+                        source.data(), source.size(), buffer1.data(), len);
+                    if (err.error) {
+                      std::cerr << "Error: at position " << err.count
+                                << " out of " << source.size() << std::endl;
+                      for (size_t i = err.count; i < source.size(); i++) {
+                        printf("0x%02x (%c) ", uint8_t(source[i]), source[i]);
+                      }
+                      printf("\n");
+                      throw std::runtime_error(
+                          "Error: is input valid base64? " +
+                          std::to_string(err.error) + " at position " +
+                          std::to_string(err.count));
+                    }
+                  }
+                });
+#endif
     }
   }
 };
@@ -620,6 +674,7 @@ void bench_bun() {
       if (!e->supported_by_runtime_system()) {
         continue;
       }
+      simdutf::get_active_implementation() = e;
       pretty_print(1, source.size(), "simdutf::" + e->name(),
                    bench([&source, &buffer1, &e, &base64_size]() {
                      base64_size = e->binary_to_base64(
